CUSTOM_R_AND_S = [
    "eduPersonTargetedID",
    "eduPersonPrincipalName",
    "mail",
    "displayName",
    "givenName",
    "sn",
    "eduPersonScopedAffiliation",
    "eduPersonUniqueId",
]

RESEARCH_AND_SCHOLARSHIP = "http://refeds.org/category/research-and-scholarship"

RELEASE = {
    "": ["eduPersonTargetedID"],
    RESEARCH_AND_SCHOLARSHIP: CUSTOM_R_AND_S,
}



#!/usr/bin/env python
import datetime
import os
import re
from unittest.mock import patch

from pathutils import full_path

from saml2 import BINDING_HTTP_ARTIFACT
from saml2 import BINDING_HTTP_POST
from saml2 import BINDING_HTTP_REDIRECT
from saml2 import BINDING_SOAP
from saml2 import config
from saml2 import sigver
from saml2.attribute_converter import ac_factory
from saml2.attribute_converter import d_to_local_name
from saml2.mdstore import MetadataStore
from saml2.mdstore import locations
from saml2.mdstore import name
from saml2.s_utils import UnknownPrincipal


TESTS_DIR = os.path.dirname(__file__)

sec_config = config.Config()
# sec_config.xmlsec_binary = sigver.get_xmlsec_binary(["/opt/local/bin"])

TEST_METADATA_STRING = """
<EntitiesDescriptor
    xmlns="urn:oasis:names:tc:SAML:2.0:metadata"
    xmlns:md="urn:oasis:names:tc:SAML:2.0:metadata"
    xmlns:shibmeta="urn:mace:shibboleth:metadata:1.0"
    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
    xmlns:ds="http://www.w3.org/2000/09/xmldsig#"
    Name="urn:mace:example.com:test-1.0">
  <EntityDescriptor
    entityID="http://xenosmilus.umdc.umu.se/simplesaml/saml2/idp/metadata.php"
    xml:base="swamid-1.0/idp.umu.se-saml2.xml">
  <IDPSSODescriptor protocolSupportEnumeration="urn:oasis:names:tc:SAML:2.0:protocol">
    <KeyDescriptor>
      <ds:KeyInfo>
        <ds:X509Data>
          <ds:X509Certificate>
            MIICsDCCAhmgAwIBAgIJAJrzqSSwmDY9MA0GCSqGSIb3DQEBBQUAMEUxCzAJBgNV
            BAYTAkFVMRMwEQYDVQQIEwpTb21lLVN0YXRlMSEwHwYDVQQKExhJbnRlcm5ldCBX
            aWRnaXRzIFB0eSBMdGQwHhcNMDkxMDA2MTk0OTQxWhcNMDkxMTA1MTk0OTQxWjBF
            MQswCQYDVQQGEwJBVTETMBEGA1UECBMKU29tZS1TdGF0ZTEhMB8GA1UEChMYSW50
            ZXJuZXQgV2lkZ2l0cyBQdHkgTHRkMIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKB
            gQDJg2cms7MqjniT8Fi/XkNHZNPbNVQyMUMXE9tXOdqwYCA1cc8vQdzkihscQMXy
            3iPw2cMggBu6gjMTOSOxECkuvX5ZCclKr8pXAJM5cY6gVOaVO2PdTZcvDBKGbiaN
            efiEw5hnoZomqZGp8wHNLAUkwtH9vjqqvxyS/vclc6k2ewIDAQABo4GnMIGkMB0G
            A1UdDgQWBBRePsKHKYJsiojE78ZWXccK9K4aJTB1BgNVHSMEbjBsgBRePsKHKYJs
            iojE78ZWXccK9K4aJaFJpEcwRTELMAkGA1UEBhMCQVUxEzARBgNVBAgTClNvbWUt
            U3RhdGUxITAfBgNVBAoTGEludGVybmV0IFdpZGdpdHMgUHR5IEx0ZIIJAJrzqSSw
            mDY9MAwGA1UdEwQFMAMBAf8wDQYJKoZIhvcNAQEFBQADgYEAJSrKOEzHO7TL5cy6
            h3qh+3+JAk8HbGBW+cbX6KBCAw/mzU8flK25vnWwXS3dv2FF3Aod0/S7AWNfKib5
            U/SA9nJaz/mWeF9S0farz9AQFc8/NSzAzaVq7YbM4F6f6N2FRl7GikdXRCed45j6
            mrPzGzk3ECbupFnqyREH3+ZPSdk=</ds:X509Certificate>
        </ds:X509Data>
      </ds:KeyInfo>
    </KeyDescriptor>
    <NameIDFormat>urn:oasis:names:tc:SAML:2.0:nameid-format:transient</NameIDFormat>
    <SingleSignOnService
        Binding="urn:oasis:names:tc:SAML:2.0:bindings:HTTP-Redirect"
        Location="http://xenosmilus.umdc.umu.se/simplesaml/saml2/idp/metadata.php"/>
  </IDPSSODescriptor>
  <Organization>
    <OrganizationName xml:lang="en">Catalogix</OrganizationName>
    <OrganizationDisplayName xml:lang="en">Catalogix</OrganizationDisplayName>
    <OrganizationURL xml:lang="en">http://www.catalogix.se</OrganizationURL>
  </Organization>
  <ContactPerson contactType="technical">
    <SurName>Hedberg</SurName>
    <EmailAddress>datordrift@catalogix.se</EmailAddress>
  </ContactPerson>
</EntityDescriptor>
</EntitiesDescriptor>
"""

ATTRCONV = ac_factory(full_path("attributemaps"))

METADATACONF = {
    "1": {"local": [full_path("swamid-1.0.xml")]},
    "2": {"local": [full_path("InCommon-metadata.xml")]},
    "3": {"local": [full_path("extended.xml")]},
    # "7": {
    #     "local": [full_path("metadata_sp_1.xml"),
    #               full_path("InCommon-metadata.xml")],
    #     "remote": [
    #         {"url": "https://kalmar2.org/simplesaml/module.php/aggregator/?id=kalmarcentral2&set=saml2",
    #          "cert": full_path("kalmar2.pem")}]
    # },
    "4": {"local": [full_path("metadata_example.xml")]},
    "5": {"local": [full_path("metadata.aaitest.xml")]},
    "8": {"mdfile": [full_path("swamid.md")]},
    "9": {"local": [full_path("metadata")]},
    "10": {
        "remote": [
            {
                "url": "http://md.incommon.org/InCommon/InCommon-metadata-export.xml",
                "cert": full_path("inc-md-cert.pem"),
            }
        ]
    },
    "11": {"inline": [TEST_METADATA_STRING]},
}


def _eq(l1, l2):
    return set(l1) == set(l2)


def _fix_valid_until(xmlstring):
    new_date = datetime.datetime.now() + datetime.timedelta(days=1)
    new_date = new_date.strftime("%Y-%m-%dT%H:%M:%SZ")
    return re.sub(r' validUntil=".*?"', f' validUntil="{new_date}"', xmlstring)


def test_swami_1():
    UMU_IDP = "https://idp.umu.se/saml2/idp/metadata.php"
    mds = MetadataStore(ATTRCONV, sec_config, disable_ssl_certificate_validation=True)

    mds.imp(METADATACONF["1"])
    assert len(mds) == 1  # One source
    idps = mds.with_descriptor("idpsso")
    assert idps.keys()
    idpsso = mds.single_sign_on_service(UMU_IDP)
    assert len(idpsso) == 1
    assert list(locations(idpsso)) == ["https://idp.umu.se/saml2/idp/SSOService.php"]

    _name = name(mds[UMU_IDP])
    assert _name == "Umeå University (SAML2)"
    certs = mds.certs(UMU_IDP, "idpsso", "signing")
    assert len(certs) == 1

    sps = mds.with_descriptor("spsso")
    assert len(sps) == 108

    wants = mds.attribute_requirement("https://connect8.sunet.se/shibboleth")
    lnamn = [d_to_local_name(mds.attrc, attr) for attr in wants["optional"]]
    assert _eq(lnamn, ["eduPersonPrincipalName", "mail", "givenName", "sn", "eduPersonScopedAffiliation"])

    wants = mds.attribute_requirement("https://beta.lobber.se/shibboleth")
    assert wants["required"] == []
    lnamn = [d_to_local_name(mds.attrc, attr) for attr in wants["optional"]]
    assert _eq(
        lnamn,
        ["eduPersonPrincipalName", "mail", "givenName", "sn", "eduPersonScopedAffiliation", "eduPersonEntitlement"],
    )


def test_incommon_1():
    mds = MetadataStore(ATTRCONV, sec_config, disable_ssl_certificate_validation=True)

    mds.imp(METADATACONF["2"])

    print(mds.entities())
    assert mds.entities() > 1700
    idps = mds.with_descriptor("idpsso")
    print(idps.keys())
    assert len(idps) > 300  # ~ 18%
    try:
        _ = mds.single_sign_on_service("urn:mace:incommon:uiuc.edu")
    except UnknownPrincipal:
        pass

    idpsso = mds.single_sign_on_service("urn:mace:incommon:alaska.edu")
    assert len(idpsso) == 1
    print(idpsso)
    assert list(locations(idpsso)) == ["https://idp.alaska.edu/idp/profile/SAML2/Redirect/SSO"]

    sps = mds.with_descriptor("spsso")

    acs_sp = []
    for nam, desc in sps.items():
        if "attribute_consuming_service" in desc:
            acs_sp.append(nam)

    assert len(acs_sp) == 0

    # Look for attribute authorities
    aas = mds.with_descriptor("attribute_authority")

    print(aas.keys())
    assert len(aas) == 180


def test_ext_2():
    mds = MetadataStore(ATTRCONV, sec_config, disable_ssl_certificate_validation=True)

    mds.imp(METADATACONF["3"])
    # No specific binding defined

    ents = mds.with_descriptor("spsso")
    for binding in [BINDING_SOAP, BINDING_HTTP_POST, BINDING_HTTP_ARTIFACT, BINDING_HTTP_REDIRECT]:
        assert mds.single_logout_service(list(ents.keys())[0], binding, "spsso")


def test_example():
    mds = MetadataStore(ATTRCONV, sec_config, disable_ssl_certificate_validation=True)

    mds.imp(METADATACONF["4"])
    assert len(mds.keys()) == 1
    idps = mds.with_descriptor("idpsso")

    assert list(idps.keys()) == ["http://xenosmilus.umdc.umu.se/simplesaml/saml2/idp/metadata.php"]
    certs = mds.certs("http://xenosmilus.umdc.umu.se/simplesaml/saml2/idp/metadata.php", "idpsso", "signing")
    assert len(certs) == 1


def test_switch_1():
    mds = MetadataStore(ATTRCONV, sec_config, disable_ssl_certificate_validation=True)

    mds.imp(METADATACONF["5"])
    assert len(mds.keys()) > 160
    idps = mds.with_descriptor("idpsso")
    print(idps.keys())
    idpsso = mds.single_sign_on_service("https://aai-demo-idp.switch.ch/idp/shibboleth")
    assert len(idpsso) == 1
    print(idpsso)
    assert list(locations(idpsso)) == ["https://aai-demo-idp.switch.ch/idp/profile/SAML2/Redirect/SSO"]
    assert len(idps) > 30
    aas = mds.with_descriptor("attribute_authority")
    print(aas.keys())
    aad = aas["https://aai-demo-idp.switch.ch/idp/shibboleth"]
    print(aad.keys())
    assert len(aad["attribute_authority_descriptor"]) == 1
    assert len(aad["idpsso_descriptor"]) == 1

    sps = mds.with_descriptor("spsso")
    dual = [eid for eid, ent in idps.items() if eid in sps]
    print(len(dual))
    assert len(dual) == 0


def test_metadata_file():
    sec_config.xmlsec_binary = sigver.get_xmlsec_binary(["/opt/local/bin"])
    mds = MetadataStore(ATTRCONV, sec_config, disable_ssl_certificate_validation=True)

    mds.imp(METADATACONF["8"])
    print(len(mds.keys()))
    assert len(mds.keys()) == 560


# def test_mdx_service():
#     sec_config.xmlsec_binary = sigver.get_xmlsec_binary(["/opt/local/bin"])
#     http = HTTPBase(verify=False, ca_bundle=None)
#
#     mdx = MetaDataMDX(quote_plus, ATTRCONV,
#                       "http://pyff-test.nordu.net",
#                       sec_config, None, http)
#     foo = mdx.service("https://idp.umu.se/saml2/idp/metadata.php",
#                       "idpsso_descriptor", "single_sign_on_service")
#
#     assert len(foo) == 1
#     assert foo.keys()[0] == BINDING_HTTP_REDIRECT
#
#
# def test_mdx_certs():
#     sec_config.xmlsec_binary = sigver.get_xmlsec_binary(["/opt/local/bin"])
#     http = HTTPBase(verify=False, ca_bundle=None)
#
#     mdx = MetaDataMDX(quote_plus, ATTRCONV,
#                       "http://pyff-test.nordu.net",
#                       sec_config, None, http)
#     foo = mdx.certs("https://idp.umu.se/saml2/idp/metadata.php", "idpsso")
#
#     assert len(foo) == 1


def test_load_local_dir():
    sec_config.xmlsec_binary = sigver.get_xmlsec_binary(["/opt/local/bin"])
    mds = MetadataStore(ATTRCONV, sec_config, disable_ssl_certificate_validation=True)

    mds.imp(METADATACONF["9"])
    print(mds)
    assert len(mds) == 3  # Three sources
    assert len(mds.keys()) == 4  # number of idps


@patch("saml2.httpbase.requests.request")
def test_load_external(mock_request):
    filepath = os.path.join(TESTS_DIR, "remote_data/InCommon-metadata-export.xml")
    with open(filepath) as fd:
        data = fd.read()
    mock_request.return_value.ok = True
    mock_request.return_value.status_code = 200
    mock_request.return_value.content = data

    sec_config.xmlsec_binary = sigver.get_xmlsec_binary(["/opt/local/bin"])
    mds = MetadataStore(ATTRCONV, sec_config, disable_ssl_certificate_validation=True)

    mds.imp(METADATACONF["10"])
    print(mds)
    assert len(mds) == 1  # One source
    assert len(mds.keys()) > 1  # number of idps


def test_load_string():
    sec_config.xmlsec_binary = sigver.get_xmlsec_binary(["/opt/local/bin"])
    mds = MetadataStore(ATTRCONV, sec_config, disable_ssl_certificate_validation=True)

    mds.imp(METADATACONF["11"])
    print(mds)
    assert len(mds.keys()) == 1
    idps = mds.with_descriptor("idpsso")

    assert list(idps.keys()) == ["http://xenosmilus.umdc.umu.se/simplesaml/saml2/idp/metadata.php"]
    certs = mds.certs("http://xenosmilus.umdc.umu.se/simplesaml/saml2/idp/metadata.php", "idpsso", "signing")
    assert len(certs) == 1


if __name__ == "__main__":
    test_load_external()



from contextlib import closing

from saml2.client import Saml2Client
from saml2.saml import AuthnContextClassRef
from saml2.server import Server


def test_authn_request_with_acs_by_index():
    # ACS index and location from SP metadata in servera.xml.
    ACS_INDEX = "4"
    ACS_LOCATION = "http://lingon.catalogix.se:8087/another/path"

    # Create SP using the configuration found in servera_conf.py.
    sp = Saml2Client(config_file="servera_conf")

    # Generate an authn request object that uses AssertionConsumerServiceIndex
    # instead of AssertionConsumerServiceURL. The index with label ACS_INDEX
    # exists in the SP metadata in servera.xml.
    request_id, authn_request = sp.create_authn_request(sp.config.entityid, assertion_consumer_service_index=ACS_INDEX)

    assert authn_request.requested_authn_context.authn_context_class_ref == [
        AuthnContextClassRef(accr)
        for accr in sp.config.getattr("requested_authn_context").get("authn_context_class_ref")
    ]
    assert authn_request.requested_authn_context.comparison == (
        sp.config.getattr("requested_authn_context").get("comparison")
    )

    # Make sure the authn_request contains AssertionConsumerServiceIndex.
    acs_index = getattr(authn_request, "assertion_consumer_service_index", None)
    assert acs_index == ACS_INDEX

    # Create IdP.
    with closing(Server(config_file="idp_all_conf")) as idp:
        # Ask the IdP to pick out the binding and destination from the
        # authn_request.
        binding, destination = idp.pick_binding("assertion_consumer_service", request=authn_request)

        # Make sure the IdP pick_binding method picks the correct location
        # or destination based on the ACS index in the authn request.
        assert destination == ACS_LOCATION



from saml2 import md
from saml2.mdie import from_dict
from saml2.mdstore import load_metadata_modules


__author__ = "rolandh"

ONTS = load_metadata_modules()


def _eq(l1, l2):
    return set(l1) == set(l2)


def _class(cls):
    return f"{cls.c_namespace}&{cls.c_tag}"


def test_construct_contact():
    c = from_dict(
        {
            "__class__": _class(md.ContactPerson),
            "given_name": {"text": "Roland", "__class__": _class(md.GivenName)},
            "sur_name": {"text": "Hedberg", "__class__": _class(md.SurName)},
            "email_address": [{"text": "roland@catalogix.se", "__class__": _class(md.EmailAddress)}],
        },
        ONTS,
    )

    print(c)
    assert c.given_name.text == "Roland"
    assert c.sur_name.text == "Hedberg"
    assert c.email_address[0].text == "roland@catalogix.se"
    assert _eq(c.keyswv(), ["given_name", "sur_name", "email_address"])



#!/usr/bin/env python
#

"""Tests for xmldsig"""

__author__ = "tmatsuo@example.com (Takashi MATSUO)"

import unittest

import ds_data

import saml2.xmldsig as ds


class TestObject:
    def setup_class(self):
        self.object = ds.Object()

    def testAccessors(self):
        """Test for Object accessors"""
        self.object.id = "object_id"
        self.object.mime_type = "test/plain; charset=UTF-8"
        self.object.encoding = ds.ENCODING_BASE64
        new_object = ds.object_from_string(self.object.to_string())
        assert new_object.id == "object_id"
        assert new_object.mime_type == "test/plain; charset=UTF-8"
        assert new_object.encoding == ds.ENCODING_BASE64

    def testUsingTestData(self):
        """Test for object_from_string() using test data"""
        new_object = ds.object_from_string(ds_data.TEST_OBJECT)
        assert new_object.id == "object_id"
        assert new_object.encoding == ds.ENCODING_BASE64
        assert new_object.text.strip() == "V2VkIEp1biAgNCAxMjoxMTowMyBFRFQgMjAwMwo"


class TestMgmtData:
    def setup_class(self):
        self.mgmt_data = ds.MgmtData()

    def testAccessors(self):
        """Test for MgmtData accessors"""
        self.mgmt_data.text = "mgmt data"
        new_mgmt_data = ds.mgmt_data_from_string(self.mgmt_data.to_string())
        assert new_mgmt_data.text.strip() == "mgmt data"

    def testUsingTestData(self):
        """Test for mgmt_data_from_string() using test data"""
        new_mgmt_data = ds.mgmt_data_from_string(ds_data.TEST_MGMT_DATA)
        assert new_mgmt_data.text.strip() == "mgmt data"


class TestSPKISexp:
    def setup_class(self):
        self.spki_sexp = ds.SPKISexp()

    def testAccessors(self):
        """Test for SPKISexp accessors"""
        self.spki_sexp.text = "spki sexp"
        new_spki_sexp = ds.spki_sexp_from_string(self.spki_sexp.to_string())
        assert new_spki_sexp.text.strip() == "spki sexp"

    def testUsingTestData(self):
        """Test for spki_sexp_from_string() using test data"""
        new_spki_sexp = ds.spki_sexp_from_string(ds_data.TEST_SPKI_SEXP)
        assert new_spki_sexp.text.strip() == "spki sexp"


class TestSPKIData:
    def setup_class(self):
        self.spki_data = ds.SPKIData()

    def testAccessors(self):
        """Test for SPKIData accessors"""
        self.spki_data.spki_sexp.append(ds.spki_sexp_from_string(ds_data.TEST_SPKI_SEXP))
        new_spki_data = ds.spki_data_from_string(self.spki_data.to_string())
        assert new_spki_data.spki_sexp[0].text.strip() == "spki sexp"

    def testUsingTestData(self):
        """Test for spki_data_from_string() using test data"""
        new_spki_data = ds.spki_data_from_string(ds_data.TEST_SPKI_DATA)
        print(new_spki_data)
        assert new_spki_data.spki_sexp[0].text.strip() == "spki sexp"
        assert new_spki_data.spki_sexp[1].text.strip() == "spki sexp2"


class TestPGPData:
    def setup_class(self):
        self.pgp_data = ds.PGPData()

    def testAccessors(self):
        """Test for PGPData accessors"""
        self.pgp_data.pgp_key_id = ds.PGPKeyID(text="pgp key id")
        self.pgp_data.pgp_key_packet = ds.PGPKeyPacket(text="pgp key packet")
        new_pgp_data = ds.pgp_data_from_string(self.pgp_data.to_string())
        assert isinstance(new_pgp_data.pgp_key_id, ds.PGPKeyID)
        assert isinstance(new_pgp_data.pgp_key_packet, ds.PGPKeyPacket)
        assert new_pgp_data.pgp_key_id.text.strip() == "pgp key id"
        assert new_pgp_data.pgp_key_packet.text.strip() == "pgp key packet"

    def testUsingTestData(self):
        """Test for pgp_data_from_string() using test data"""
        new_pgp_data = ds.pgp_data_from_string(ds_data.TEST_PGP_DATA)
        assert isinstance(new_pgp_data.pgp_key_id, ds.PGPKeyID)
        assert isinstance(new_pgp_data.pgp_key_packet, ds.PGPKeyPacket)
        assert new_pgp_data.pgp_key_id.text.strip() == "pgp key id"
        assert new_pgp_data.pgp_key_packet.text.strip() == "pgp key packet"


class TestX509IssuerSerial:
    def setup_class(self):
        self.x509_issuer_serial = ds.X509IssuerSerialType_()

    def testAccessors(self):
        """Test for X509SerialNumber accessors"""
        self.x509_issuer_serial.x509_issuer_name = ds.X509IssuerName(text="issuer name")
        self.x509_issuer_serial.x509_serial_number = ds.X509SerialNumber(text="1")
        new_x509_issuer_serial = ds.x509_issuer_serial_type__from_string(self.x509_issuer_serial.to_string())
        assert new_x509_issuer_serial.x509_issuer_name.text.strip() == "issuer name"
        assert new_x509_issuer_serial.x509_serial_number.text.strip() == "1"

    def testUsingTestData(self):
        """Test for x509_issuer_serial_from_string() using test data"""
        new_x509_issuer_serial = ds.x509_issuer_serial_from_string(ds_data.TEST_X509_ISSUER_SERIAL)
        assert new_x509_issuer_serial.x509_issuer_name.text.strip() == "issuer name"
        assert new_x509_issuer_serial.x509_serial_number.text.strip() == "1"


class TestX509Data:
    def setup_class(self):
        self.x509_data = ds.X509Data()

    def testAccessors(self):
        """Test for X509Data accessors"""
        st = ds.x509_issuer_serial_from_string(ds_data.TEST_X509_ISSUER_SERIAL)
        print(st)
        self.x509_data.x509_issuer_serial = st
        self.x509_data.x509_ski = ds.X509SKI(text="x509 ski")
        self.x509_data.x509_subject_name = ds.X509SubjectName(text="x509 subject name")
        self.x509_data.x509_certificate = ds.X509Certificate(text="x509 certificate")
        self.x509_data.x509_crl = ds.X509CRL(text="x509 crl")

        new_x509_data = ds.x509_data_from_string(self.x509_data.to_string())
        print(new_x509_data.keyswv())
        print(new_x509_data.__dict__.keys())
        assert new_x509_data.x509_issuer_serial
        assert isinstance(new_x509_data.x509_issuer_serial, ds.X509IssuerSerial)
        assert new_x509_data.x509_ski.text.strip() == "x509 ski"
        assert isinstance(new_x509_data.x509_ski, ds.X509SKI)
        assert new_x509_data.x509_subject_name.text.strip() == "x509 subject name"
        assert isinstance(new_x509_data.x509_subject_name, ds.X509SubjectName)
        assert new_x509_data.x509_certificate.text.strip() == "x509 certificate"
        assert isinstance(new_x509_data.x509_certificate, ds.X509Certificate)
        assert new_x509_data.x509_crl.text.strip() == "x509 crl"
        assert isinstance(new_x509_data.x509_crl, ds.X509CRL)

    def testUsingTestData(self):
        """Test for x509_data_from_string() using test data"""
        new_x509_data = ds.x509_data_from_string(ds_data.TEST_X509_DATA)
        assert isinstance(new_x509_data.x509_issuer_serial, ds.X509IssuerSerial)
        assert new_x509_data.x509_ski.text.strip() == "x509 ski"
        assert isinstance(new_x509_data.x509_ski, ds.X509SKI)
        assert new_x509_data.x509_subject_name.text.strip() == "x509 subject name"
        assert isinstance(new_x509_data.x509_subject_name, ds.X509SubjectName)
        assert new_x509_data.x509_certificate.text.strip() == "x509 certificate"
        assert isinstance(new_x509_data.x509_certificate, ds.X509Certificate)
        assert new_x509_data.x509_crl.text.strip() == "x509 crl"
        assert isinstance(new_x509_data.x509_crl, ds.X509CRL)


class TestTransform:
    def setup_class(self):
        self.transform = ds.Transform()

    def testAccessors(self):
        """Test for Transform accessors"""
        self.transform.x_path.append(ds.TransformType_XPath(text="xpath"))
        self.transform.algorithm = ds.TRANSFORM_ENVELOPED
        new_transform = ds.transform_from_string(self.transform.to_string())
        assert isinstance(new_transform.x_path[0], ds.TransformType_XPath)
        assert new_transform.x_path[0].text.strip() == "xpath"
        assert new_transform.algorithm == ds.TRANSFORM_ENVELOPED

    def testUsingTestData(self):
        """Test for transform_from_string() using test data"""
        new_transform = ds.transform_from_string(ds_data.TEST_TRANSFORM)
        assert isinstance(new_transform.x_path[0], ds.TransformType_XPath)
        assert new_transform.x_path[0].text.strip() == "xpath"
        assert new_transform.algorithm == ds.TRANSFORM_ENVELOPED


class TestTransforms:
    def setup_class(self):
        self.transforms = ds.Transforms()

    def testAccessors(self):
        """Test for Transforms accessors"""
        self.transforms.transform.append(ds.transform_from_string(ds_data.TEST_TRANSFORM))
        self.transforms.transform.append(ds.transform_from_string(ds_data.TEST_TRANSFORM))
        new_transforms = ds.transforms_from_string(self.transforms.to_string())
        assert isinstance(new_transforms.transform[0], ds.Transform)
        assert isinstance(new_transforms.transform[1], ds.Transform)
        assert new_transforms.transform[0].algorithm == ds.TRANSFORM_ENVELOPED
        assert new_transforms.transform[1].algorithm == ds.TRANSFORM_ENVELOPED
        assert new_transforms.transform[0].x_path[0].text.strip() == "xpath"
        assert new_transforms.transform[1].x_path[0].text.strip() == "xpath"

    def testUsingTestData(self):
        """Test for transform_from_string() using test data"""
        new_transforms = ds.transforms_from_string(ds_data.TEST_TRANSFORMS)
        assert isinstance(new_transforms.transform[0], ds.Transform)
        assert isinstance(new_transforms.transform[1], ds.Transform)
        assert new_transforms.transform[0].algorithm == ds.TRANSFORM_ENVELOPED
        assert new_transforms.transform[1].algorithm == ds.TRANSFORM_ENVELOPED
        assert new_transforms.transform[0].x_path[0].text.strip() == "xpath"
        assert new_transforms.transform[1].x_path[0].text.strip() == "xpath"


class TestRetrievalMethod:
    def setup_class(self):
        self.retrieval_method = ds.RetrievalMethod()

    def testAccessors(self):
        """Test for RetrievalMethod accessors"""
        self.retrieval_method.uri = "http://www.example.com/URI"
        self.retrieval_method.type = "http://www.example.com/Type"
        self.retrieval_method.transforms = ds.transforms_from_string(ds_data.TEST_TRANSFORMS)
        new_retrieval_method = ds.retrieval_method_from_string(self.retrieval_method.to_string())
        assert new_retrieval_method.uri == "http://www.example.com/URI"
        assert new_retrieval_method.type == "http://www.example.com/Type"
        assert isinstance(new_retrieval_method.transforms, ds.Transforms)

    def testUsingTestData(self):
        """Test for retrieval_method_from_string() using test data"""
        new_retrieval_method = ds.retrieval_method_from_string(ds_data.TEST_RETRIEVAL_METHOD)
        assert new_retrieval_method.uri == "http://www.example.com/URI"
        assert new_retrieval_method.type == "http://www.example.com/Type"
        assert isinstance(new_retrieval_method.transforms, ds.Transforms)


class TestRSAKeyValue:
    def setup_class(self):
        self.rsa_key_value = ds.RSAKeyValue()

    def testAccessors(self):
        """Test for RSAKeyValue accessors"""
        self.rsa_key_value.modulus = ds.Modulus(text="modulus")
        self.rsa_key_value.exponent = ds.Exponent(text="exponent")
        new_rsa_key_value = ds.rsa_key_value_from_string(self.rsa_key_value.to_string())
        assert isinstance(new_rsa_key_value.modulus, ds.Modulus)
        assert isinstance(new_rsa_key_value.exponent, ds.Exponent)
        assert new_rsa_key_value.modulus.text.strip() == "modulus"
        assert new_rsa_key_value.exponent.text.strip() == "exponent"

    def testUsingTestData(self):
        """Test for rsa_key_value_from_string() using test data"""
        new_rsa_key_value = ds.rsa_key_value_from_string(ds_data.TEST_RSA_KEY_VALUE)
        assert isinstance(new_rsa_key_value.modulus, ds.Modulus)
        assert isinstance(new_rsa_key_value.exponent, ds.Exponent)
        assert new_rsa_key_value.modulus.text.strip() == "modulus"
        assert new_rsa_key_value.exponent.text.strip() == "exponent"


class TestDSAKeyValue:
    def setup_class(self):
        self.dsa_key_value = ds.DSAKeyValue()

    def testAccessors(self):
        """Test for DSAKeyValue accessors"""
        self.dsa_key_value.p = ds.P(text="p")
        self.dsa_key_value.q = ds.Q(text="q")
        self.dsa_key_value.g = ds.G(text="g")
        self.dsa_key_value.y = ds.Y(text="y")
        self.dsa_key_value.j = ds.J(text="j")
        self.dsa_key_value.seed = ds.Seed(text="seed")
        self.dsa_key_value.pgen_counter = ds.PgenCounter(text="pgen counter")
        new_dsa_key_value = ds.dsa_key_value_from_string(self.dsa_key_value.to_string())
        assert isinstance(new_dsa_key_value.p, ds.P)
        assert isinstance(new_dsa_key_value.q, ds.Q)
        assert isinstance(new_dsa_key_value.g, ds.G)
        assert isinstance(new_dsa_key_value.y, ds.Y)
        assert isinstance(new_dsa_key_value.j, ds.J)
        assert isinstance(new_dsa_key_value.seed, ds.Seed)
        assert isinstance(new_dsa_key_value.pgen_counter, ds.PgenCounter)
        assert new_dsa_key_value.p.text.strip() == "p"
        assert new_dsa_key_value.q.text.strip() == "q"
        assert new_dsa_key_value.g.text.strip() == "g"
        assert new_dsa_key_value.y.text.strip() == "y"
        assert new_dsa_key_value.j.text.strip() == "j"
        assert new_dsa_key_value.seed.text.strip() == "seed"
        assert new_dsa_key_value.pgen_counter.text.strip() == "pgen counter"

    def testUsingTestData(self):
        """Test for dsa_key_value_from_string() using test data"""
        new_dsa_key_value = ds.dsa_key_value_from_string(ds_data.TEST_DSA_KEY_VALUE)
        assert isinstance(new_dsa_key_value.p, ds.P)
        assert isinstance(new_dsa_key_value.q, ds.Q)
        assert isinstance(new_dsa_key_value.g, ds.G)
        assert isinstance(new_dsa_key_value.y, ds.Y)
        assert isinstance(new_dsa_key_value.j, ds.J)
        assert isinstance(new_dsa_key_value.seed, ds.Seed)
        assert isinstance(new_dsa_key_value.pgen_counter, ds.PgenCounter)
        assert new_dsa_key_value.p.text.strip() == "p"
        assert new_dsa_key_value.q.text.strip() == "q"
        assert new_dsa_key_value.g.text.strip() == "g"
        assert new_dsa_key_value.y.text.strip() == "y"
        assert new_dsa_key_value.j.text.strip() == "j"
        assert new_dsa_key_value.seed.text.strip() == "seed"
        assert new_dsa_key_value.pgen_counter.text.strip() == "pgen counter"


class TestKeyValue:
    def setup_class(self):
        self.key_value = ds.KeyValue()

    def testAccessors(self):
        """Test for KeyValue accessors"""
        self.key_value.dsa_key_value = ds.dsa_key_value_from_string(ds_data.TEST_DSA_KEY_VALUE)
        new_key_value = ds.key_value_from_string(self.key_value.to_string())
        assert isinstance(new_key_value.dsa_key_value, ds.DSAKeyValue)
        self.key_value.dsa_key_value = None
        self.key_value.rsa_key_value = ds.rsa_key_value_from_string(ds_data.TEST_RSA_KEY_VALUE)
        new_key_value = ds.key_value_from_string(self.key_value.to_string())
        assert isinstance(new_key_value.rsa_key_value, ds.RSAKeyValue)

    def testUsingTestData(self):
        """Test for key_value_from_string() using test data"""
        new_key_value = ds.key_value_from_string(ds_data.TEST_KEY_VALUE1)
        assert isinstance(new_key_value.dsa_key_value, ds.DSAKeyValue)
        self.key_value.dsa_key_value = None
        self.key_value.rsa_key_value = ds.rsa_key_value_from_string(ds_data.TEST_RSA_KEY_VALUE)
        new_key_value = ds.key_value_from_string(ds_data.TEST_KEY_VALUE2)
        assert isinstance(new_key_value.rsa_key_value, ds.RSAKeyValue)


class TestKeyName:
    def setup_class(self):
        self.key_name = ds.KeyName()

    def testAccessors(self):
        """Test for KeyName accessors"""
        self.key_name.text = "key name"
        new_key_name = ds.key_name_from_string(self.key_name.to_string())
        assert new_key_name.text.strip() == "key name"

    def testUsingTestData(self):
        """Test for key_name_from_string() using test data"""
        new_key_name = ds.key_name_from_string(ds_data.TEST_KEY_NAME)
        assert new_key_name.text.strip() == "key name"


class TestKeyInfo:
    def setup_class(self):
        self.key_info = ds.KeyInfo()

    def testAccessors(self):
        """Test for KeyInfo accessors"""
        self.key_info.key_name.append(ds.key_name_from_string(ds_data.TEST_KEY_NAME))
        self.key_info.key_value.append(ds.key_value_from_string(ds_data.TEST_KEY_VALUE1))
        self.key_info.retrieval_method.append(ds.retrieval_method_from_string(ds_data.TEST_RETRIEVAL_METHOD))
        self.key_info.x509_data.append(ds.x509_data_from_string(ds_data.TEST_X509_DATA))
        self.key_info.pgp_data.append(ds.pgp_data_from_string(ds_data.TEST_PGP_DATA))
        self.key_info.spki_data.append(ds.spki_data_from_string(ds_data.TEST_SPKI_DATA))
        self.key_info.mgmt_data.append(ds.mgmt_data_from_string(ds_data.TEST_MGMT_DATA))
        self.key_info.id = "id"
        new_key_info = ds.key_info_from_string(self.key_info.to_string())

        assert isinstance(new_key_info.key_name[0], ds.KeyName)
        assert isinstance(new_key_info.key_value[0], ds.KeyValue)
        assert isinstance(new_key_info.retrieval_method[0], ds.RetrievalMethod)
        assert isinstance(new_key_info.x509_data[0], ds.X509Data)
        assert isinstance(new_key_info.pgp_data[0], ds.PGPData)
        assert isinstance(new_key_info.spki_data[0], ds.SPKIData)
        assert isinstance(new_key_info.mgmt_data[0], ds.MgmtData)
        assert new_key_info.id == "id"

    def testUsingTestData(self):
        """Test for key_info_from_string() using test data"""
        new_key_info = ds.key_info_from_string(ds_data.TEST_KEY_INFO)
        assert isinstance(new_key_info.key_name[0], ds.KeyName)
        assert isinstance(new_key_info.key_value[0], ds.KeyValue)
        assert isinstance(new_key_info.retrieval_method[0], ds.RetrievalMethod)
        assert isinstance(new_key_info.x509_data[0], ds.X509Data)
        assert isinstance(new_key_info.pgp_data[0], ds.PGPData)
        assert isinstance(new_key_info.spki_data[0], ds.SPKIData)
        assert isinstance(new_key_info.mgmt_data[0], ds.MgmtData)
        assert new_key_info.id == "id"


class TestDigestValue:
    def setup_class(self):
        self.digest_value = ds.DigestValue()

    def testAccessors(self):
        """Test for DigestValue accessors"""
        self.digest_value.text = "digest value"
        new_digest_value = ds.digest_value_from_string(self.digest_value.to_string())
        assert new_digest_value.text.strip() == "digest value"

    def testUsingTestData(self):
        """Test for digest_value_from_string() using test data"""
        new_digest_value = ds.digest_value_from_string(ds_data.TEST_DIGEST_VALUE)
        assert new_digest_value.text.strip() == "digest value"


class TestDigestMethod:
    def setup_class(self):
        self.digest_method = ds.DigestMethod()

    def testAccessors(self):
        """Test for DigestMethod accessors"""
        self.digest_method.algorithm = ds.DIGEST_SHA1
        new_digest_method = ds.digest_method_from_string(self.digest_method.to_string())
        assert new_digest_method.algorithm == ds.DIGEST_SHA1

    def testUsingTestData(self):
        """Test for digest_method_from_string() using test data"""
        new_digest_method = ds.digest_method_from_string(ds_data.TEST_DIGEST_METHOD)
        assert new_digest_method.algorithm == ds.DIGEST_SHA1


class TestReference:
    def setup_class(self):
        self.reference = ds.Reference()

    def testAccessors(self):
        """Test for Reference accessors"""
        self.reference.transforms = ds.transforms_from_string(ds_data.TEST_TRANSFORMS)
        self.reference.digest_method = ds.digest_method_from_string(ds_data.TEST_DIGEST_METHOD)
        self.reference.digest_value = ds.digest_value_from_string(ds_data.TEST_DIGEST_VALUE)
        self.reference.id = "id"
        self.reference.uri = "http://www.example.com/URI"
        self.reference.type = "http://www.example.com/Type"
        new_reference = ds.reference_from_string(self.reference.to_string())
        assert isinstance(new_reference.transforms, ds.Transforms)
        assert isinstance(new_reference.digest_method, ds.DigestMethod)
        assert isinstance(new_reference.digest_value, ds.DigestValue)
        assert new_reference.id == "id"
        assert new_reference.uri == "http://www.example.com/URI"
        assert new_reference.type == "http://www.example.com/Type"

    def testUsingTestData(self):
        """Test for reference_from_string() using test data"""
        new_reference = ds.reference_from_string(ds_data.TEST_REFERENCE)
        assert isinstance(new_reference.transforms, ds.Transforms)
        assert isinstance(new_reference.digest_method, ds.DigestMethod)
        assert isinstance(new_reference.digest_value, ds.DigestValue)
        assert new_reference.id == "id"
        assert new_reference.uri == "http://www.example.com/URI"
        assert new_reference.type == "http://www.example.com/Type"


class TestSignatureMethod:
    def setup_class(self):
        self.signature_method = ds.SignatureMethod()

    def testAccessors(self):
        """Test for SignatureMethod accessors"""
        self.signature_method.algorithm = ds.SIG_RSA_SHA1
        self.signature_method.hmac_output_length = ds.HMACOutputLength(text="8")
        new_signature_method = ds.signature_method_from_string(self.signature_method.to_string())
        assert isinstance(new_signature_method.hmac_output_length, ds.HMACOutputLength)
        assert new_signature_method.hmac_output_length.text.strip() == "8"
        assert new_signature_method.algorithm == ds.SIG_RSA_SHA1

    def testUsingTestData(self):
        """Test for signature_method_from_string() using test data"""
        new_signature_method = ds.signature_method_from_string(ds_data.TEST_SIGNATURE_METHOD)
        assert isinstance(new_signature_method.hmac_output_length, ds.HMACOutputLength)
        assert new_signature_method.hmac_output_length.text.strip() == "8"
        assert new_signature_method.algorithm == ds.SIG_RSA_SHA1


class TestCanonicalizationMethod:
    def setup_class(self):
        self.canonicalization_method = ds.CanonicalizationMethod()

    def testAccessors(self):
        """Test for CanonicalizationMethod accessors"""
        self.canonicalization_method.algorithm = ds.TRANSFORM_C14N_WITH_COMMENTS
        new_canonicalization_method = ds.canonicalization_method_from_string(self.canonicalization_method.to_string())
        assert new_canonicalization_method.algorithm == ds.TRANSFORM_C14N_WITH_COMMENTS

    def testUsingTestData(self):
        """Test for canonicalization_method_from_string() using test data"""
        new_canonicalization_method = ds.canonicalization_method_from_string(ds_data.TEST_CANONICALIZATION_METHOD)
        assert new_canonicalization_method.algorithm == "http://www.w3.org/TR/2001/REC-xml-c14n-20010315#WithComments"


class TestSignedInfo:
    def setup_class(self):
        self.si = ds.SignedInfo()

    def testAccessors(self):
        """Test for SignedInfo accessors"""
        self.si.id = "id"
        self.si.canonicalization_method = ds.canonicalization_method_from_string(ds_data.TEST_CANONICALIZATION_METHOD)
        self.si.signature_method = ds.signature_method_from_string(ds_data.TEST_SIGNATURE_METHOD)
        self.si.reference.append(ds.reference_from_string(ds_data.TEST_REFERENCE))
        new_si = ds.signed_info_from_string(self.si.to_string())
        assert new_si.id == "id"
        assert isinstance(new_si.canonicalization_method, ds.CanonicalizationMethod)
        assert isinstance(new_si.signature_method, ds.SignatureMethod)
        assert isinstance(new_si.reference[0], ds.Reference)

    def testUsingTestData(self):
        """Test for signed_info_from_string() using test data"""
        new_si = ds.signed_info_from_string(ds_data.TEST_SIGNED_INFO)
        assert new_si.id == "id"
        assert isinstance(new_si.canonicalization_method, ds.CanonicalizationMethod)
        assert isinstance(new_si.signature_method, ds.SignatureMethod)
        assert isinstance(new_si.reference[0], ds.Reference)


class TestSignatureValue:
    def setup_class(self):
        self.signature_value = ds.SignatureValue()

    def testAccessors(self):
        """Test for SignatureValue accessors"""
        self.signature_value.id = "id"
        self.signature_value.text = "signature value"
        new_signature_value = ds.signature_value_from_string(self.signature_value.to_string())
        assert new_signature_value.id == "id"
        assert new_signature_value.text.strip() == "signature value"

    def testUsingTestData(self):
        """Test for signature_value_from_string() using test data"""
        new_signature_value = ds.signature_value_from_string(ds_data.TEST_SIGNATURE_VALUE)
        assert new_signature_value.id == "id"
        assert new_signature_value.text.strip() == "signature value"


class TestSignature:
    def setup_class(self):
        self.signature = ds.Signature()

    def testAccessors(self):
        """Test for Signature accessors"""
        self.signature.id = "id"
        self.signature.signed_info = ds.signed_info_from_string(ds_data.TEST_SIGNED_INFO)
        self.signature.signature_value = ds.signature_value_from_string(ds_data.TEST_SIGNATURE_VALUE)
        self.signature.key_info = ds.key_info_from_string(ds_data.TEST_KEY_INFO)
        self.signature.object.append(ds.object_from_string(ds_data.TEST_OBJECT))

        new_signature = ds.signature_from_string(self.signature.to_string())
        assert new_signature.id == "id"
        assert isinstance(new_signature.signed_info, ds.SignedInfo)
        assert isinstance(new_signature.signature_value, ds.SignatureValue)
        assert isinstance(new_signature.key_info, ds.KeyInfo)
        assert isinstance(new_signature.object[0], ds.Object)

    def testUsingTestData(self):
        """Test for signature_value_from_string() using test data"""
        new_signature = ds.signature_from_string(ds_data.TEST_SIGNATURE)
        assert new_signature.id == "id"
        assert isinstance(new_signature.signed_info, ds.SignedInfo)
        assert isinstance(new_signature.signature_value, ds.SignatureValue)
        assert isinstance(new_signature.key_info, ds.KeyInfo)
        assert isinstance(new_signature.object[0], ds.Object)


if __name__ == "__main__":
    unittest.main()



from pathutils import full_path

from saml2 import config
from saml2.attribute_converter import ac_factory
from saml2.filter import AllowDescriptor
from saml2.mdstore import MetadataStore


__author__ = "roland"

sec_config = config.Config()


ATTRCONV = ac_factory(full_path("attributemaps"))

METADATACONF = {
    "1": [
        {
            "class": "saml2.mdstore.MetaDataFile",
            "metadata": [(full_path("swamid-2.0.xml"),)],
        }
    ],
}


def test_swamid_sp():
    mds = MetadataStore(
        ATTRCONV, sec_config, disable_ssl_certificate_validation=True, filter=AllowDescriptor(["spsso"])
    )

    mds.imp(METADATACONF["1"])
    sps = mds.with_descriptor("spsso")
    assert len(sps) == 417
    idps = mds.with_descriptor("idpsso")
    assert idps == {}


def test_swamid_idp():
    mds = MetadataStore(
        ATTRCONV, sec_config, disable_ssl_certificate_validation=True, filter=AllowDescriptor(["idpsso"])
    )

    mds.imp(METADATACONF["1"])
    sps = mds.with_descriptor("spsso")
    assert len(sps) == 0
    idps = mds.with_descriptor("idpsso")
    assert len(idps) == 275


if __name__ == "__main__":
    test_swamid_idp()



#!/usr/bin/env python

from base64 import encodebytes as b64encode

import pytest

from saml2.authn_context import INTERNETPROTOCOLPASSWORD
from saml2.saml import NAMEID_FORMAT_TRANSIENT
from saml2.samlp import NameIDPolicy
from saml2.server import Server


try:
    from saml2.s2repoze.plugins.sp import make_plugin
except ImportError:
    make_plugin = None

ENV1 = {
    "SERVER_SOFTWARE": "CherryPy/3.1.2 WSGI Server",
    "SCRIPT_NAME": "",
    "ACTUAL_SERVER_PROTOCOL": "HTTP/1.1",
    "REQUEST_METHOD": "GET",
    "PATH_INFO": "/krissms",
    "SERVER_PROTOCOL": "HTTP/1.1",
    "QUERY_STRING": "",
    "REMOTE_ADDR": "127.0.0.1",
    "HTTP_USER_AGENT": "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_2; en-us) ",
    "HTTP_CONNECTION": "keep-alive",
    "SERVER_NAME": "lingon-catalogix-se-2.local",
    "REMOTE_PORT": "57309",
    "wsgi.url_scheme": "http",
    "SERVER_PORT": "8087",
    "HTTP_HOST": "127.0.0.1:8087",
    "wsgi.multithread": True,
    "HTTP_ACCEPT": "application/xml,application/xhtml+xml,text/html;q=0.9," "text/plain;q=0.8,image/png,*/*;q=0.5",
    "wsgi.version": (1, 0),
    "wsgi.run_once": False,
    "wsgi.multiprocess": False,
    "HTTP_ACCEPT_LANGUAGE": "en-us",
    "HTTP_ACCEPT_ENCODING": "gzip, deflate",
}

trans_name_policy = NameIDPolicy(format=NAMEID_FORMAT_TRANSIENT, allow_create="true")

AUTHN = {"class_ref": INTERNETPROTOCOLPASSWORD, "authn_auth": "http://www.example.com/login"}


@pytest.mark.skipif(not make_plugin, reason="s2repoze dependencies not installed")
class TestSP:
    def setup_class(self):
        self.sp = make_plugin("rem", saml_conf="server_conf")
        # Explicitly allow unsigned responses for this test
        self.sp.saml_client.want_response_signed = False
        self.server = Server(config_file="idp_conf")

    def teardown_class(self):
        self.server.close()

    def test_setup(self):
        assert self.sp

    def test_identify(self):
        # Create a SAMLResponse
        ava = {"givenName": ["Derek"], "surName": ["Jeter"], "mail": ["derek@nyy.mlb.com"], "title": ["The man"]}

        resp_str = "%s" % self.server.create_authn_response(
            ava,
            "id1",
            "http://lingon.catalogix.se:8087/",
            "urn:mace:example.com:saml:roland:sp",
            trans_name_policy,
            "foba0001@example.com",
            authn=AUTHN,
        )

        resp_str = b64encode(resp_str.encode("utf-8"))
        self.sp.outstanding_queries = {"id1": "http://www.example.com/service"}
        session_info = self.sp._eval_authn_response({}, {"SAMLResponse": [resp_str]})

        assert len(session_info) > 1
        assert session_info["came_from"] == "http://www.example.com/service"
        assert session_info["ava"] == {
            "givenName": ["Derek"],
            "mail": ["derek@nyy.mlb.com"],
            "sn": ["Jeter"],
            "title": ["The man"],
        }


if __name__ == "__main__":
    _sp = TestSP()
    _sp.setup_class()
    _sp.test_identify()



import saml2
from saml2 import BINDING_HTTP_POST
from saml2 import extension_elements_to_elements
from saml2 import md
from saml2.attribute_converter import ac_factory
from saml2.config import IdPConfig
from saml2.config import SPConfig
from saml2.extension import idpdisc
from saml2.extension import mdui
from saml2.extension import shibmd
from saml2.saml import NAME_FORMAT_URI


def _eq(l1, l2):
    return set(l1) == set(l2)


SP = {
    "name": "Rolands SP",
    "description": "One of the best SPs in business",
    "service": {
        "sp": {
            "endpoints": {
                "single_logout_service": ["http://localhost:8087/logout"],
                "assertion_consumer_service": [
                    {"location": "http://localhost:8087/", "binding": BINDING_HTTP_POST},
                ],
            },
            "required_attributes": ["sn", "givenName", "mail"],
            "optional_attributes": ["title"],
            "idp": {
                "": "https://example.com/saml2/idp/SSOService.php",
            },
        }
    },
    "metadata": {
        "local": ["foo.xml"],
    },
    "attribute_map_dir": "attributemaps",
}

IDP = {
    "name": "Rolands IdP",
    "service": {
        "idp": {
            "endpoints": {
                "single_sign_on_service": ["http://localhost:8088/sso"],
            },
            "policy": {
                "default": {
                    "lifetime": {"minutes": 15},
                    "attribute_restrictions": None,  # means all I have
                    "name_form": "urn:oasis:names:tc:SAML:2.0:attrname-format:uri",
                },
                "urn:mace:example.com:saml:roland:sp": {
                    "lifetime": {"minutes": 5},
                    "nameid_format": "urn:oasis:names:tc:SAML:2.0:nameid-format:persistent",
                },
            },
            "scope": ["example.org"],
            "ui_info": {
                "privacy_statement_url": "http://example.com/saml2/privacyStatement.html",
                "information_url": "http://example.com/saml2/info.html",
                "logo": {"height": "40", "width": "30", "text": "http://example.com/logo.jpg"},
                "display_name": "Example Co.",
                "description": {"text": "Exempel bolag", "lang": "se"},
                "keywords": {"lang": "en", "text": ["foo", "bar"]},
            },
        }
    },
    "metadata": {
        "local": ["bar.xml"],
    },
}


def test_org_1():
    desc = {
        "name": [
            ("Example Company", "en"),
            ("Exempel AB", "se"),
            "Example",
        ],
        "display_name": ["Example AS", ("Voorbeeld AZ", "")],
        "url": [("http://example.com", "en")],
    }
    org = metadata.do_organization_info(desc)
    print(org)
    assert isinstance(org, md.Organization)
    print(org.keyswv())
    assert _eq(org.keyswv(), ["organization_name", "organization_display_name", "organization_url"])
    assert len(org.organization_name) == 3
    assert len(org.organization_display_name) == 2
    assert len(org.organization_url) == 1


def test_org_2():
    desc = {
        "name": [
            ("Example Company", "en"),
            ("Exempel AB", "se"),
            "Example",
        ],
        "display_name": "Example AS",
        "url": ("http://example.com", "en"),
    }
    org = metadata.do_organization_info(desc)
    print(org)
    assert _eq(org.keyswv(), ["organization_name", "organization_display_name", "organization_url"])
    assert len(org.organization_name) == 3
    assert len(org.organization_display_name) == 1
    assert org.organization_display_name[0].text == "Example AS"
    assert len(org.organization_url) == 1
    assert isinstance(org.organization_url[0], md.OrganizationURL)
    assert org.organization_url[0].lang == "en"
    assert org.organization_url[0].text == "http://example.com"


def test_org_3():
    desc = {"display_name": ["Rolands SAML"]}
    org = metadata.do_organization_info(desc)
    assert _eq(org.keyswv(), ["organization_display_name"])
    assert len(org.organization_display_name) == 1


def test_contact_0():
    conf = [
        {
            "given_name": "Roland",
            "sur_name": "Hedberg",
            "telephone_number": "+46 70 100 00 00",
            "email_address": ["foo@eample.com", "foo@example.org"],
            "contact_type": "technical",
        }
    ]
    contact_person = metadata.do_contact_person_info(conf)
    assert _eq(
        contact_person[0].keyswv(), ["given_name", "sur_name", "contact_type", "telephone_number", "email_address"]
    )
    print(contact_person[0])
    person = contact_person[0]
    assert person.contact_type == "technical"
    assert isinstance(person.given_name, md.GivenName)
    assert person.given_name.text == "Roland"
    assert isinstance(person.sur_name, md.SurName)
    assert person.sur_name.text == "Hedberg"
    assert isinstance(person.telephone_number[0], md.TelephoneNumber)
    assert person.telephone_number[0].text == "+46 70 100 00 00"
    assert len(person.email_address) == 2
    assert isinstance(person.email_address[0], md.EmailAddress)
    assert person.email_address[0].text == "foo@eample.com"


def test_do_endpoints():
    eps = metadata.do_endpoints(SP["service"]["sp"]["endpoints"], metadata.ENDPOINTS["sp"])
    print(eps)
    assert _eq(eps.keys(), ["assertion_consumer_service", "single_logout_service"])

    assert len(eps["single_logout_service"]) == 1
    sls = eps["single_logout_service"][0]
    assert sls.location == "http://localhost:8087/logout"
    assert sls.binding == BINDING_HTTP_POST

    assert len(eps["assertion_consumer_service"]) == 1
    acs = eps["assertion_consumer_service"][0]
    assert acs.location == "http://localhost:8087/"
    assert acs.binding == BINDING_HTTP_POST

    assert "artifact_resolution_service" not in eps
    assert "manage_name_id_service" not in eps


def test_required_attributes():
    attrconverters = ac_factory("../tests/attributemaps")
    ras = metadata.do_requested_attribute(
        SP["service"]["sp"]["required_attributes"], attrconverters, is_required="true"
    )
    assert len(ras) == len(SP["service"]["sp"]["required_attributes"])
    print(ras[0])
    assert ras[0].name == "urn:oid:2.5.4.4"
    assert ras[0].name_format == NAME_FORMAT_URI
    assert ras[0].is_required == "true"


def test_optional_attributes():
    attrconverters = ac_factory("../tests/attributemaps")
    ras = metadata.do_requested_attribute(SP["service"]["sp"]["optional_attributes"], attrconverters)
    assert len(ras) == len(SP["service"]["sp"]["optional_attributes"])
    print(ras[0])
    assert ras[0].name == "urn:oid:2.5.4.12"
    assert ras[0].name_format == NAME_FORMAT_URI
    assert ras[0].is_required == "false"


def test_do_sp_sso_descriptor():
    conf = SPConfig().load(SP)
    spsso = metadata.do_spsso_descriptor(conf)

    assert isinstance(spsso, md.SPSSODescriptor)
    assert _eq(
        spsso.keyswv(),
        [
            "authn_requests_signed",
            "attribute_consuming_service",
            "single_logout_service",
            "protocol_support_enumeration",
            "assertion_consumer_service",
            "want_assertions_signed",
        ],
    )

    assert spsso.authn_requests_signed == "false"
    assert spsso.want_assertions_signed == "true"
    assert len(spsso.attribute_consuming_service) == 1
    acs = spsso.attribute_consuming_service[0]
    print(acs.keyswv())
    assert _eq(acs.keyswv(), ["requested_attribute", "service_name", "service_description", "index"])
    assert acs.service_name[0].text == SP["name"]
    assert acs.service_description[0].text == SP["description"]
    assert len(acs.requested_attribute) == 4
    assert acs.requested_attribute[0].friendly_name == "sn"
    assert acs.requested_attribute[0].name == "urn:oid:2.5.4.4"
    assert acs.requested_attribute[0].name_format == NAME_FORMAT_URI
    assert acs.requested_attribute[0].is_required == "true"


def test_do_sp_sso_descriptor_2():
    SP["service"]["sp"]["discovery_response"] = "http://example.com/sp/ds"

    conf = SPConfig().load(SP)
    spsso = metadata.do_spsso_descriptor(conf)

    assert isinstance(spsso, md.SPSSODescriptor)
    print(spsso.keyswv())
    assert _eq(
        spsso.keyswv(),
        [
            "authn_requests_signed",
            "attribute_consuming_service",
            "single_logout_service",
            "protocol_support_enumeration",
            "assertion_consumer_service",
            "want_assertions_signed",
            "extensions",
        ],
    )

    exts = spsso.extensions.extension_elements
    assert len(exts) == 1
    print(exts)
    idpd = saml2.extension_element_to_element(exts[0], idpdisc.ELEMENT_FROM_STRING, namespace=idpdisc.NAMESPACE)
    print(idpd)
    assert idpd.location == "http://example.com/sp/ds"
    assert idpd.index == "0"
    assert idpd.binding == "urn:oasis:names:tc:SAML:profiles:SSO:idp-discovery-protocol"


def test_entity_description():
    # confd = eval(open("../tests/server.config").read())
    confd = SPConfig().load_file("server_conf")
    print(confd.attribute_converters)
    entd = metadata.entity_descriptor(confd)
    assert entd is not None
    print(entd.keyswv())
    assert _eq(entd.keyswv(), ["valid_until", "entity_id", "contact_person", "spsso_descriptor", "organization"])
    print(entd)
    assert entd.entity_id == "urn:mace:example.com:saml:roland:sp"


def test_do_idp_sso_descriptor():
    conf = IdPConfig().load(IDP)
    idpsso = metadata.do_idpsso_descriptor(conf)

    assert isinstance(idpsso, md.IDPSSODescriptor)
    assert _eq(
        idpsso.keyswv(),
        ["protocol_support_enumeration", "single_sign_on_service", "want_authn_requests_signed", "extensions"],
    )
    exts = idpsso.extensions.extension_elements
    assert len(exts) == 2
    print(exts)
    inst = saml2.extension_element_to_element(exts[0], shibmd.ELEMENT_FROM_STRING, namespace=shibmd.NAMESPACE)
    assert isinstance(inst, shibmd.Scope)
    assert inst.text == "example.org"
    assert inst.regexp == "false"

    uiinfo = saml2.extension_element_to_element(exts[1], mdui.ELEMENT_FROM_STRING, namespace=mdui.NAMESPACE)

    assert uiinfo
    assert _eq(
        uiinfo.keyswv(), ["display_name", "description", "information_url", "privacy_statement_url", "keywords", "logo"]
    )

    assert len(uiinfo.privacy_statement_url) == 1
    assert uiinfo.privacy_statement_url[0].text == "http://example.com/saml2/privacyStatement.html"
    assert len(uiinfo.description) == 1
    assert uiinfo.description[0].text == "Exempel bolag"
    assert uiinfo.description[0].lang == "se"

    res = extension_elements_to_elements(exts, [shibmd, mdui])

    assert len(res) == 2
    # one is a shibmd.Scope instance and the other a mdui.UIInfo instance
    if isinstance(res[0], shibmd.Scope):
        assert isinstance(res[1], mdui.UIInfo)
    elif isinstance(res[1], shibmd.Scope):
        assert isinstance(res[0], mdui.UIInfo)

    found = idpsso.extensions.find_extensions(mdui.UIInfo.c_tag, mdui.NAMESPACE)
    assert len(found) == 1

    elem = idpsso.extensions.extensions_as_elements(mdui.UIInfo.c_tag, mdui)
    assert len(elem) == 1
    assert isinstance(elem[0], mdui.UIInfo)



#!/usr/bin/env python
from pathutils import full_path

from saml2 import BINDING_HTTP_ARTIFACT
from saml2 import BINDING_HTTP_POST
from saml2 import BINDING_HTTP_REDIRECT
from saml2 import BINDING_SOAP
from saml2 import BINDING_URI
from saml2.saml import NAME_FORMAT_URI
from saml2.saml import NAMEID_FORMAT_PERSISTENT


BASE = "http://localhost:8088"

CONFIG = {
    "entityid": f"{BASE}/saml/idp",
    "name": "Rolands IdP",
    "service": {
        "aa": {
            "endpoints": {"attribute_service": [(f"{BASE}/aap", BINDING_HTTP_POST), (f"{BASE}/aas", BINDING_SOAP)]},
        },
        "aq": {
            "endpoints": {"authn_query_service": [(f"{BASE}/aqs", BINDING_SOAP)]},
        },
        "idp": {
            "endpoints": {
                "single_sign_on_service": [
                    (f"{BASE}/sso/redirect", BINDING_HTTP_REDIRECT),
                    (f"{BASE}/sso/post", BINDING_HTTP_POST),
                    (f"{BASE}/sso/art", BINDING_HTTP_ARTIFACT),
                    (f"{BASE}/sso/paos", BINDING_SOAP),
                ],
                "single_logout_service": [
                    (f"{BASE}/slo/soap", BINDING_SOAP),
                    (f"{BASE}/slo/post", BINDING_HTTP_POST),
                ],
                "artifact_resolution_service": [(f"{BASE}/ars", BINDING_SOAP)],
                "assertion_id_request_service": [(f"{BASE}/airs", BINDING_URI)],
                "authn_query_service": [(f"{BASE}/aqs", BINDING_SOAP)],
                "manage_name_id_service": [
                    (f"{BASE}/mni/soap", BINDING_SOAP),
                    (f"{BASE}/mni/post", BINDING_HTTP_POST),
                    (f"{BASE}/mni/redirect", BINDING_HTTP_REDIRECT),
                    (f"{BASE}/mni/art", BINDING_HTTP_ARTIFACT),
                ],
                "name_id_mapping_service": [
                    (f"{BASE}/nim/soap", BINDING_SOAP),
                    (f"{BASE}/nim/post", BINDING_HTTP_POST),
                    (f"{BASE}/nim/redirect", BINDING_HTTP_REDIRECT),
                    (f"{BASE}/nim/art", BINDING_HTTP_ARTIFACT),
                ],
            },
            "policy": {
                "default": {
                    "lifetime": {"minutes": 15},
                    "attribute_restrictions": None,  # means all I have
                    "name_form": NAME_FORMAT_URI,
                },
                "urn:mace:example.com:saml:roland:sp": {
                    "lifetime": {"minutes": 5},
                    "nameid_format": NAMEID_FORMAT_PERSISTENT,
                    # "attribute_restrictions":{
                    #     "givenName": None,
                    #     "surName": None,
                    # }
                },
            },
            "subject_data": ("mongodb", "subject"),
            "session_storage": ("mongodb", "session"),
        },
    },
    "debug": 1,
    "key_file": full_path("test.key"),
    "cert_file": full_path("test.pem"),
    # "xmlsec_binary": None,
    "xmlsec_path": ["/opt/local/bin", "usr/local/bin"],
    "metadata": [
        {
            "class": "saml2.mdstore.MetaDataFile",
            "metadata": [(full_path("servera.xml"),), (full_path("vo_metadata.xml"),)],
        }
    ],
    "attribute_map_dir": full_path("attributemaps"),
    "organization": {
        "name": "Exempel AB",
        "display_name": [("Exempel ÄB", "se"), ("Example Co.", "en")],
        "url": "http://www.example.com/roland",
    },
    "contact_person": [
        {
            "given_name": "John",
            "sur_name": "Smith",
            "email_address": ["john.smith@example.com"],
            "contact_type": "technical",
        },
    ],
}



#!/usr/bin/env python
from saml2.saml import AUTHN_PASSWORD


__author__ = "rolandh"

import json


BASE = "http://localhost:8087"
# BASE= "http://lingon.catalogix.se:8087"

metadata = open("./sp/sp.xml").read()

AUTHN = {"class_ref": AUTHN_PASSWORD, "authn_auth": "http://lingon.catalogix.se/login"}

info = {
    "start_page": BASE,
    "entity_id": f"{BASE}/sp.xml",
    "result": {
        "matches": {"content": "<h2>Your identity are"},
    },
    "metadata": metadata,
    "args": {
        "AuthnResponse": {
            "sign_assertion": "always",  # always, never
            "sign_response": "never",  # always, never
            "sign_digest_alg": ds.DIGEST_SHA256,
            "sign_signature_alg": ds.SIG_RSA_SHA256,
            "authn": AUTHN,
        }
    },
    # This is the set of attributes and values that are returned in the
    # SAML Assertion
    "identity": {"given_name": "Roland", "sn": "Hedberg"},
    # This is the value of the NameID that is return in the Subject in the
    # Assertion
    "userid": "roland",
    # regex pattern that must be contained in the resulting echo page to validate
    # that the SP returned the right page after Login.
    "echopageIdPattern": r"<title>SAML Echo Service</title>",
    # list of regex patterns that must be contained in the resulting echo page to validate
    # that the SP's echo page returns expected SAMLe response values (e.g. attribute values)
    "echopageContentPattern": [
        r"Given Name\s*</td>\s*<td>Roland</td>",
        r"Userid\s*</td>\s*<td>roalnd</td>",
        r"Surname\s*</td>\s*<td>Hedberg</td>",
    ],
    "constraints": {
        "authnRequest_signature_required": True,
        # allowed for assertion & response signature:
        "signature_algorithm": [
            # ds.SIG_RSA_SHA1,  # you may need this for legacy deployments
            ds.SIG_RSA_SHA224,
            ds.SIG_RSA_SHA256,
            ds.SIG_RSA_SHA384,
            ds.SIG_RSA_SHA512,
        ],
        "digest_algorithm": [
            # ds.DIGEST_SHA1,   # you may need this for legacy deployments
            ds.DIGEST_SHA1,
            ds.DIGEST_SHA224,
            ds.DIGEST_SHA256,
            ds.DIGEST_SHA384,
            ds.DIGEST_SHA512,
            ds.DIGEST_RIPEMD160,
        ],
    },
}

print(json.dumps(info))



from saml2 import BINDING_HTTP_REDIRECT
from saml2 import BINDING_SOAP
from saml2.saml import NAME_FORMAT_URI


BASE = "http://localhost:8089/"

from pathutils import full_path


CONFIG = {
    "entityid": "urn:mace:example.com:saml:roland:idpr",
    "name": "Rolands restrictied IdP",
    "service": {
        "idp": {
            "endpoints": {
                "single_sign_on_service": [(f"{BASE}sso", BINDING_HTTP_REDIRECT)],
                "attribute_service": [(f"{BASE}aa", BINDING_SOAP)],
            },
            "policy": {
                "default": {"lifetime": {"minutes": 15}, "name_form": NAME_FORMAT_URI},
                "urn:mace:example.com:saml:roland:sp": {
                    "lifetime": {"minutes": 5},
                    "attribute_restrictions": {
                        "givenName": None,
                        "surName": None,
                        "mail": [".*@example.com"],
                        "eduPersonAffiliation": ["(employee|staff|faculty)"],
                    },
                },
            },
            "subject_data": full_path("subject_data.db"),
        }
    },
    "key_file": full_path("test.key"),
    "cert_file": full_path("test.pem"),
    "xmlsec_binary": None,
    "metadata": [
        {
            "class": "saml2.mdstore.MetaDataFile",
            "metadata": [(full_path("sp_0.metadata"),)],
        }
    ],
    "attribute_map_dir": full_path("attributemaps"),
}



#!/usr/bin/env python

import argparse

from saml2.attribute_converter import ac_factory
from saml2.httpbase import HTTPBase
from saml2.mdstore import MetaDataExtern
from saml2.mdstore import MetaDataFile
from saml2.sigver import SecurityContext
from saml2.sigver import _get_xmlsec_cryptobackend


__author__ = "rolandh"

"""
A script that imports and verifies metadata.
"""


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("-t", dest="type")
    parser.add_argument("-u", dest="url")
    parser.add_argument("-c", dest="cert")
    parser.add_argument("-a", dest="attrsmap")
    parser.add_argument("-o", dest="output")
    parser.add_argument("-x", dest="xmlsec")
    parser.add_argument("-i", dest="ignore_valid", action="store_true")
    parser.add_argument(dest="item")
    args = parser.parse_args()

    metad = None

    if args.ignore_valid:
        kwargs = {"check_validity": False}
    else:
        kwargs = {}

    if args.type == "local":
        if args.cert and args.xmlsec:
            crypto = _get_xmlsec_cryptobackend(args.xmlsec)
            sc = SecurityContext(crypto)
            metad = MetaDataFile(args.item, args.item, cert=args.cert, security=sc, **kwargs)
        else:
            metad = MetaDataFile(args.item, args.item, **kwargs)
    elif args.type == "external":
        ATTRCONV = ac_factory(args.attrsmap)
        httpc = HTTPBase()
        crypto = _get_xmlsec_cryptobackend(args.xmlsec)
        sc = SecurityContext(crypto)
        metad = MetaDataExtern(ATTRCONV, args.url, sc, cert=args.cert, http=httpc, **kwargs)

    if metad:
        metad.load()
        print("OK")


if __name__ == "__main__":
    main()



#!/usr/bin/env python

#
# Generated Mon Jun 27 09:54:22 2011 by parse_xsd.py version 0.4.
#

import saml2
from saml2 import SamlBase
from saml2 import md


NAMESPACE = "urn:oasis:names:tc:SAML:metadata:rpi"


class RegistrationPolicy(md.LocalizedURIType_):
    """The urn:oasis:names:tc:SAML:metadata:rpi:RegistrationPolicy element"""

    c_tag = "RegistrationPolicy"
    c_namespace = NAMESPACE
    c_children = md.LocalizedURIType_.c_children.copy()
    c_attributes = md.LocalizedURIType_.c_attributes.copy()
    c_child_order = md.LocalizedURIType_.c_child_order[:]
    c_cardinality = md.LocalizedURIType_.c_cardinality.copy()


def registration_policy_from_string(xml_string):
    return saml2.create_class_from_xml_string(RegistrationPolicy, xml_string)


class UsagePolicy(md.LocalizedURIType_):
    """The urn:oasis:names:tc:SAML:metadata:rpi:UsagePolicy element"""

    c_tag = "UsagePolicy"
    c_namespace = NAMESPACE
    c_children = md.LocalizedURIType_.c_children.copy()
    c_attributes = md.LocalizedURIType_.c_attributes.copy()
    c_child_order = md.LocalizedURIType_.c_child_order[:]
    c_cardinality = md.LocalizedURIType_.c_cardinality.copy()


def usage_policy_from_string(xml_string):
    return saml2.create_class_from_xml_string(UsagePolicy, xml_string)


class PublicationType_(SamlBase):
    """The urn:oasis:names:tc:SAML:metadata:rpi:PublicationType element"""

    c_tag = "PublicationType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["publisher"] = ("publisher", "string", True)
    c_attributes["creationInstant"] = ("creation_instant", "dateTime", False)
    c_attributes["publicationId"] = ("publication_id", "string", False)

    def __init__(
        self,
        publisher=None,
        creation_instant=None,
        publication_id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.publisher = publisher
        self.creation_instant = creation_instant
        self.publication_id = publication_id


def publication_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(PublicationType_, xml_string)


class RegistrationInfoType_(SamlBase):
    """The urn:oasis:names:tc:SAML:metadata:rpi:RegistrationInfoType element"""

    c_tag = "RegistrationInfoType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:metadata:rpi}RegistrationPolicy"] = (
        "registration_policy",
        [RegistrationPolicy],
    )
    c_cardinality["registration_policy"] = {"min": 0}
    c_attributes["registrationAuthority"] = ("registration_authority", "string", True)
    c_attributes["registrationInstant"] = ("registration_instant", "dateTime", False)
    c_child_order.extend(["registration_policy"])

    def __init__(
        self,
        registration_policy=None,
        registration_authority=None,
        registration_instant=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.registration_policy = registration_policy or []
        self.registration_authority = registration_authority
        self.registration_instant = registration_instant


def registration_info_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(RegistrationInfoType_, xml_string)


class PublicationInfoType_(SamlBase):
    """The urn:oasis:names:tc:SAML:metadata:rpi:PublicationInfoType element"""

    c_tag = "PublicationInfoType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:metadata:rpi}UsagePolicy"] = ("usage_policy", [UsagePolicy])
    c_cardinality["usage_policy"] = {"min": 0}
    c_attributes["publisher"] = ("publisher", "string", True)
    c_attributes["creationInstant"] = ("creation_instant", "dateTime", False)
    c_attributes["publicationId"] = ("publication_id", "string", False)
    c_child_order.extend(["usage_policy"])

    def __init__(
        self,
        usage_policy=None,
        publisher=None,
        creation_instant=None,
        publication_id=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.usage_policy = usage_policy or []
        self.publisher = publisher
        self.creation_instant = creation_instant
        self.publication_id = publication_id


def publication_info_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(PublicationInfoType_, xml_string)


class Publication(PublicationType_):
    """The urn:oasis:names:tc:SAML:metadata:rpi:Publication element"""

    c_tag = "Publication"
    c_namespace = NAMESPACE
    c_children = PublicationType_.c_children.copy()
    c_attributes = PublicationType_.c_attributes.copy()
    c_child_order = PublicationType_.c_child_order[:]
    c_cardinality = PublicationType_.c_cardinality.copy()


def publication_from_string(xml_string):
    return saml2.create_class_from_xml_string(Publication, xml_string)


class RegistrationInfo(RegistrationInfoType_):
    """The urn:oasis:names:tc:SAML:metadata:rpi:RegistrationInfo element"""

    c_tag = "RegistrationInfo"
    c_namespace = NAMESPACE
    c_children = RegistrationInfoType_.c_children.copy()
    c_attributes = RegistrationInfoType_.c_attributes.copy()
    c_child_order = RegistrationInfoType_.c_child_order[:]
    c_cardinality = RegistrationInfoType_.c_cardinality.copy()


def registration_info_from_string(xml_string):
    return saml2.create_class_from_xml_string(RegistrationInfo, xml_string)


class PublicationInfo(PublicationInfoType_):
    """The urn:oasis:names:tc:SAML:metadata:rpi:PublicationInfo element"""

    c_tag = "PublicationInfo"
    c_namespace = NAMESPACE
    c_children = PublicationInfoType_.c_children.copy()
    c_attributes = PublicationInfoType_.c_attributes.copy()
    c_child_order = PublicationInfoType_.c_child_order[:]
    c_cardinality = PublicationInfoType_.c_cardinality.copy()


def publication_info_from_string(xml_string):
    return saml2.create_class_from_xml_string(PublicationInfo, xml_string)


class PublicationPathType_(SamlBase):
    """The urn:oasis:names:tc:SAML:metadata:rpi:PublicationPathType element"""

    c_tag = "PublicationPathType"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_children["{urn:oasis:names:tc:SAML:metadata:rpi}Publication"] = ("publication", [Publication])
    c_cardinality["publication"] = {"min": 0}
    c_child_order.extend(["publication"])

    def __init__(self, publication=None, text=None, extension_elements=None, extension_attributes=None):
        SamlBase.__init__(
            self, text=text, extension_elements=extension_elements, extension_attributes=extension_attributes
        )
        self.publication = publication or []


def publication_path_type__from_string(xml_string):
    return saml2.create_class_from_xml_string(PublicationPathType_, xml_string)


class PublicationPath(PublicationPathType_):
    """The urn:oasis:names:tc:SAML:metadata:rpi:PublicationPath element"""

    c_tag = "PublicationPath"
    c_namespace = NAMESPACE
    c_children = PublicationPathType_.c_children.copy()
    c_attributes = PublicationPathType_.c_attributes.copy()
    c_child_order = PublicationPathType_.c_child_order[:]
    c_cardinality = PublicationPathType_.c_cardinality.copy()


def publication_path_from_string(xml_string):
    return saml2.create_class_from_xml_string(PublicationPath, xml_string)


ELEMENT_FROM_STRING = {
    RegistrationInfo.c_tag: registration_info_from_string,
    RegistrationInfoType_.c_tag: registration_info_type__from_string,
    RegistrationPolicy.c_tag: registration_policy_from_string,
    PublicationInfo.c_tag: publication_info_from_string,
    PublicationInfoType_.c_tag: publication_info_type__from_string,
    UsagePolicy.c_tag: usage_policy_from_string,
    PublicationPath.c_tag: publication_path_from_string,
    PublicationPathType_.c_tag: publication_path_type__from_string,
    Publication.c_tag: publication_from_string,
    PublicationType_.c_tag: publication_type__from_string,
}

ELEMENT_BY_TAG = {
    "RegistrationInfo": RegistrationInfo,
    "RegistrationInfoType": RegistrationInfoType_,
    "RegistrationPolicy": RegistrationPolicy,
    "PublicationInfo": PublicationInfo,
    "PublicationInfoType": PublicationInfoType_,
    "UsagePolicy": UsagePolicy,
    "PublicationPath": PublicationPath,
    "PublicationPathType": PublicationPathType_,
    "Publication": Publication,
    "PublicationType": PublicationType_,
}


def factory(tag, **kwargs):
    return ELEMENT_BY_TAG[tag](**kwargs)



import logging
import os.path
import time
from time import strftime
import urllib


__author__ = "rhoerbe"

logger = logging.getLogger(__name__)


def fetch_metadata(url, path, maxage=600):
    """
    :param url:  metadata remote location
    :param path: metdata file name
    :param maxage: if max age of existing metadata file (s) is exceeded,
     the file will be fetched from the remote location
    """
    fetch = False
    if not os.path.isfile(path):
        fetch = True
        logger.debug("metadata file %s not found", path)
    elif (os.path.getmtime(path) + maxage) < time.time():
        fetch = True
        logger.debug(
            "metadata file %s from %s is more than %s s old",
            path,
            strftime("%Y-%m-%d %H:%M:%S", time.localtime(os.path.getmtime(path))),
            maxage,
        )
    else:
        logger.debug("metadata file %s is less than %s s old", path, maxage)
    if fetch:
        f = urllib.URLopener()
        try:
            f.retrieve(url, path)
            logger.debug("downloaded metadata from %s into %s", url, path)
        except Exception as e:
            logger.debug("downloaded metadata from %s failed: %s", url, str(e))



#!/usr/bin/env python

#
# Generated Fri May 27 17:23:42 2011 by parse_xsd.py version 0.4.
#

import saml2
from saml2 import SamlBase
from saml2.schema import wsdl


NAMESPACE = "http://schemas.xmlsoap.org/wsdl/soap/"


class EncodingStyle_(SamlBase):
    """The http://schemas.xmlsoap.org/wsdl/soap/:encodingStyle element"""

    c_tag = "encodingStyle"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def encoding_style__from_string(xml_string):
    return saml2.create_class_from_xml_string(EncodingStyle_, xml_string)


class TStyleChoice_(SamlBase):
    """The http://schemas.xmlsoap.org/wsdl/soap/:tStyleChoice element"""

    c_tag = "tStyleChoice"
    c_namespace = NAMESPACE
    c_value_type = {"base": "xs:string", "enumeration": ["rpc", "document"]}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def t_style_choice__from_string(xml_string):
    return saml2.create_class_from_xml_string(TStyleChoice_, xml_string)


class TOperation_(wsdl.TExtensibilityElement_):
    """The http://schemas.xmlsoap.org/wsdl/soap/:tOperation element"""

    c_tag = "tOperation"
    c_namespace = NAMESPACE
    c_children = wsdl.TExtensibilityElement_.c_children.copy()
    c_attributes = wsdl.TExtensibilityElement_.c_attributes.copy()
    c_child_order = wsdl.TExtensibilityElement_.c_child_order[:]
    c_cardinality = wsdl.TExtensibilityElement_.c_cardinality.copy()
    c_attributes["soapAction"] = ("soap_action", "anyURI", False)
    c_attributes["style"] = ("style", TStyleChoice_, False)

    def __init__(
        self,
        soap_action=None,
        style=None,
        required=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        wsdl.TExtensibilityElement_.__init__(
            self,
            required=required,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.soap_action = soap_action
        self.style = style


def t_operation__from_string(xml_string):
    return saml2.create_class_from_xml_string(TOperation_, xml_string)


class UseChoice_(SamlBase):
    """The http://schemas.xmlsoap.org/wsdl/soap/:useChoice element"""

    c_tag = "useChoice"
    c_namespace = NAMESPACE
    c_value_type = {"base": "xs:string", "enumeration": ["literal", "encoded"]}
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()


def use_choice__from_string(xml_string):
    return saml2.create_class_from_xml_string(UseChoice_, xml_string)


class TFaultRes_(SamlBase):
    """The http://schemas.xmlsoap.org/wsdl/soap/:tFaultRes element"""

    c_tag = "tFaultRes"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["{http://schemas.xmlsoap.org/wsdl/}required"] = ("required", "None", False)
    c_attributes["parts"] = ("parts", "NMTOKENS", False)
    c_attributes["encodingStyle"] = ("encoding_style", EncodingStyle_, False)
    c_attributes["use"] = ("use", UseChoice_, False)
    c_attributes["namespace"] = ("namespace", "anyURI", False)

    def __init__(
        self,
        required=None,
        parts=None,
        encoding_style=None,
        use=None,
        namespace=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.required = required
        self.parts = parts
        self.encoding_style = encoding_style
        self.use = use
        self.namespace = namespace


class TFault_(TFaultRes_):
    """The http://schemas.xmlsoap.org/wsdl/soap/:tFault element"""

    c_tag = "tFault"
    c_namespace = NAMESPACE
    c_children = TFaultRes_.c_children.copy()
    c_attributes = TFaultRes_.c_attributes.copy()
    c_child_order = TFaultRes_.c_child_order[:]
    c_cardinality = TFaultRes_.c_cardinality.copy()
    c_attributes["name"] = ("name", "NCName", True)

    def __init__(
        self,
        name=None,
        required=None,
        parts=None,
        encoding_style=None,
        use=None,
        namespace=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        TFaultRes_.__init__(
            self,
            required=required,
            parts=parts,
            encoding_style=encoding_style,
            use=use,
            namespace=namespace,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.name = name


def t_fault__from_string(xml_string):
    return saml2.create_class_from_xml_string(TFault_, xml_string)


class THeaderFault_(SamlBase):
    """The http://schemas.xmlsoap.org/wsdl/soap/:tHeaderFault element"""

    c_tag = "tHeaderFault"
    c_namespace = NAMESPACE
    c_children = SamlBase.c_children.copy()
    c_attributes = SamlBase.c_attributes.copy()
    c_child_order = SamlBase.c_child_order[:]
    c_cardinality = SamlBase.c_cardinality.copy()
    c_attributes["message"] = ("message", "QName", True)
    c_attributes["part"] = ("part", "NMTOKEN", True)
    c_attributes["use"] = ("use", UseChoice_, True)
    c_attributes["encodingStyle"] = ("encoding_style", EncodingStyle_, False)
    c_attributes["namespace"] = ("namespace", "anyURI", False)

    def __init__(
        self,
        message=None,
        part=None,
        use=None,
        encoding_style=None,
        namespace=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        SamlBase.__init__(
            self,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.message = message
        self.part = part
        self.use = use
        self.encoding_style = encoding_style
        self.namespace = namespace


def t_header_fault__from_string(xml_string):
    return saml2.create_class_from_xml_string(THeaderFault_, xml_string)


class TAddress_(wsdl.TExtensibilityElement_):
    """The http://schemas.xmlsoap.org/wsdl/soap/:tAddress element"""

    c_tag = "tAddress"
    c_namespace = NAMESPACE
    c_children = wsdl.TExtensibilityElement_.c_children.copy()
    c_attributes = wsdl.TExtensibilityElement_.c_attributes.copy()
    c_child_order = wsdl.TExtensibilityElement_.c_child_order[:]
    c_cardinality = wsdl.TExtensibilityElement_.c_cardinality.copy()
    c_attributes["location"] = ("location", "anyURI", True)

    def __init__(
        self,
        location=None,
        required=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        wsdl.TExtensibilityElement_.__init__(
            self,
            required=required,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.location = location


def t_address__from_string(xml_string):
    return saml2.create_class_from_xml_string(TAddress_, xml_string)


class TBinding_(wsdl.TExtensibilityElement_):
    """The http://schemas.xmlsoap.org/wsdl/soap/:tBinding element"""

    c_tag = "tBinding"
    c_namespace = NAMESPACE
    c_children = wsdl.TExtensibilityElement_.c_children.copy()
    c_attributes = wsdl.TExtensibilityElement_.c_attributes.copy()
    c_child_order = wsdl.TExtensibilityElement_.c_child_order[:]
    c_cardinality = wsdl.TExtensibilityElement_.c_cardinality.copy()
    c_attributes["transport"] = ("transport", "anyURI", True)
    c_attributes["style"] = ("style", TStyleChoice_, False)

    def __init__(
        self,
        transport=None,
        style=None,
        required=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        wsdl.TExtensibilityElement_.__init__(
            self,
            required=required,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.transport = transport
        self.style = style


def t_binding__from_string(xml_string):
    return saml2.create_class_from_xml_string(TBinding_, xml_string)


class Operation(TOperation_):
    """The http://schemas.xmlsoap.org/wsdl/soap/:operation element"""

    c_tag = "operation"
    c_namespace = NAMESPACE
    c_children = TOperation_.c_children.copy()
    c_attributes = TOperation_.c_attributes.copy()
    c_child_order = TOperation_.c_child_order[:]
    c_cardinality = TOperation_.c_cardinality.copy()


def operation_from_string(xml_string):
    return saml2.create_class_from_xml_string(Operation, xml_string)


class TBody_(wsdl.TExtensibilityElement_):
    """The http://schemas.xmlsoap.org/wsdl/soap/:tBody element"""

    c_tag = "tBody"
    c_namespace = NAMESPACE
    c_children = wsdl.TExtensibilityElement_.c_children.copy()
    c_attributes = wsdl.TExtensibilityElement_.c_attributes.copy()
    c_child_order = wsdl.TExtensibilityElement_.c_child_order[:]
    c_cardinality = wsdl.TExtensibilityElement_.c_cardinality.copy()
    c_attributes["parts"] = ("parts", "NMTOKENS", False)
    c_attributes["encodingStyle"] = ("encoding_style", EncodingStyle_, False)
    c_attributes["use"] = ("use", UseChoice_, False)
    c_attributes["namespace"] = ("namespace", "anyURI", False)

    def __init__(
        self,
        parts=None,
        encoding_style=None,
        use=None,
        namespace=None,
        required=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        wsdl.TExtensibilityElement_.__init__(
            self,
            required=required,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.parts = parts
        self.encoding_style = encoding_style
        self.use = use
        self.namespace = namespace


def t_body__from_string(xml_string):
    return saml2.create_class_from_xml_string(TBody_, xml_string)


class Fault(TFault_):
    """The http://schemas.xmlsoap.org/wsdl/soap/:fault element"""

    c_tag = "fault"
    c_namespace = NAMESPACE
    c_children = TFault_.c_children.copy()
    c_attributes = TFault_.c_attributes.copy()
    c_child_order = TFault_.c_child_order[:]
    c_cardinality = TFault_.c_cardinality.copy()


def fault_from_string(xml_string):
    return saml2.create_class_from_xml_string(Fault, xml_string)


class Headerfault(THeaderFault_):
    """The http://schemas.xmlsoap.org/wsdl/soap/:headerfault element"""

    c_tag = "headerfault"
    c_namespace = NAMESPACE
    c_children = THeaderFault_.c_children.copy()
    c_attributes = THeaderFault_.c_attributes.copy()
    c_child_order = THeaderFault_.c_child_order[:]
    c_cardinality = THeaderFault_.c_cardinality.copy()


def headerfault_from_string(xml_string):
    return saml2.create_class_from_xml_string(Headerfault, xml_string)


class Address(TAddress_):
    """The http://schemas.xmlsoap.org/wsdl/soap/:address element"""

    c_tag = "address"
    c_namespace = NAMESPACE
    c_children = TAddress_.c_children.copy()
    c_attributes = TAddress_.c_attributes.copy()
    c_child_order = TAddress_.c_child_order[:]
    c_cardinality = TAddress_.c_cardinality.copy()


def address_from_string(xml_string):
    return saml2.create_class_from_xml_string(Address, xml_string)


class Binding(TBinding_):
    """The http://schemas.xmlsoap.org/wsdl/soap/:binding element"""

    c_tag = "binding"
    c_namespace = NAMESPACE
    c_children = TBinding_.c_children.copy()
    c_attributes = TBinding_.c_attributes.copy()
    c_child_order = TBinding_.c_child_order[:]
    c_cardinality = TBinding_.c_cardinality.copy()


def binding_from_string(xml_string):
    return saml2.create_class_from_xml_string(Binding, xml_string)


class Body(TBody_):
    """The http://schemas.xmlsoap.org/wsdl/soap/:body element"""

    c_tag = "body"
    c_namespace = NAMESPACE
    c_children = TBody_.c_children.copy()
    c_attributes = TBody_.c_attributes.copy()
    c_child_order = TBody_.c_child_order[:]
    c_cardinality = TBody_.c_cardinality.copy()


def body_from_string(xml_string):
    return saml2.create_class_from_xml_string(Body, xml_string)


class THeader_(wsdl.TExtensibilityElement_):
    """The http://schemas.xmlsoap.org/wsdl/soap/:tHeader element"""

    c_tag = "tHeader"
    c_namespace = NAMESPACE
    c_children = wsdl.TExtensibilityElement_.c_children.copy()
    c_attributes = wsdl.TExtensibilityElement_.c_attributes.copy()
    c_child_order = wsdl.TExtensibilityElement_.c_child_order[:]
    c_cardinality = wsdl.TExtensibilityElement_.c_cardinality.copy()
    c_children["{http://schemas.xmlsoap.org/wsdl/soap/}headerfault"] = ("headerfault", [Headerfault])
    c_cardinality["headerfault"] = {"min": 0}
    c_attributes["message"] = ("message", "QName", True)
    c_attributes["part"] = ("part", "NMTOKEN", True)
    c_attributes["use"] = ("use", UseChoice_, True)
    c_attributes["encodingStyle"] = ("encoding_style", EncodingStyle_, False)
    c_attributes["namespace"] = ("namespace", "anyURI", False)
    c_child_order.extend(["headerfault"])

    def __init__(
        self,
        headerfault=None,
        message=None,
        part=None,
        use=None,
        encoding_style=None,
        namespace=None,
        required=None,
        text=None,
        extension_elements=None,
        extension_attributes=None,
    ):
        wsdl.TExtensibilityElement_.__init__(
            self,
            required=required,
            text=text,
            extension_elements=extension_elements,
            extension_attributes=extension_attributes,
        )
        self.headerfault = headerfault or []
        self.message = message
        self.part = part
        self.use = use
        self.encoding_style = encoding_style
        self.namespace = namespace


def t_header__from_string(xml_string):
    return saml2.create_class_from_xml_string(THeader_, xml_string)


class Header(THeader_):
    """The http://schemas.xmlsoap.org/wsdl/soap/:header element"""

    c_tag = "header"
    c_namespace = NAMESPACE
    c_children = THeader_.c_children.copy()
    c_attributes = THeader_.c_attributes.copy()
    c_child_order = THeader_.c_child_order[:]
    c_cardinality = THeader_.c_cardinality.copy()


def header_from_string(xml_string):
    return saml2.create_class_from_xml_string(Header, xml_string)


AG_tBodyAttributes = [
    ("encodingStyle", EncodingStyle_, False),
    ("use", UseChoice_, False),
    ("namespace", "anyURI", False),
]

AG_tHeaderAttributes = [
    ("message", "QName", True),
    ("part", "NMTOKEN", True),
    ("use", UseChoice_, True),
    ("encodingStyle", EncodingStyle_, False),
    ("namespace", "anyURI", False),
]

ELEMENT_FROM_STRING = {
    EncodingStyle_.c_tag: encoding_style__from_string,
    Binding.c_tag: binding_from_string,
    TBinding_.c_tag: t_binding__from_string,
    TStyleChoice_.c_tag: t_style_choice__from_string,
    Operation.c_tag: operation_from_string,
    TOperation_.c_tag: t_operation__from_string,
    Body.c_tag: body_from_string,
    TBody_.c_tag: t_body__from_string,
    UseChoice_.c_tag: use_choice__from_string,
    Fault.c_tag: fault_from_string,
    TFault_.c_tag: t_fault__from_string,
    Header.c_tag: header_from_string,
    THeader_.c_tag: t_header__from_string,
    Headerfault.c_tag: headerfault_from_string,
    THeaderFault_.c_tag: t_header_fault__from_string,
    Address.c_tag: address_from_string,
    TAddress_.c_tag: t_address__from_string,
}

ELEMENT_BY_TAG = {
    "encodingStyle": EncodingStyle_,
    "binding": Binding,
    "tBinding": TBinding_,
    "tStyleChoice": TStyleChoice_,
    "operation": Operation,
    "tOperation": TOperation_,
    "body": Body,
    "tBody": TBody_,
    "useChoice": UseChoice_,
    "fault": Fault,
    "tFault": TFault_,
    "header": Header,
    "tHeader": THeader_,
    "headerfault": Headerfault,
    "tHeaderFault": THeaderFault_,
    "address": Address,
    "tAddress": TAddress_,
    "tFaultRes": TFaultRes_,
}


def factory(tag, **kwargs):
    return ELEMENT_BY_TAG[tag](**kwargs)
